@@ -23,25 +23,19 @@ class OsaAPI(object):
     """
     max_try = CxConfig.CxConfig.config.max_try
     base_url = CxConfig.CxConfig.config.url
-    osa_scans_url = base_url + "/osa/scans"
-    osa_scan_by_scan_id_url = base_url + "/osa/scans/{scanId}"
-    osa_file_extensions_url = base_url + "/osa/fileextensions"
-    osa_licenses_url = base_url + "/osa/licenses"
-    osa_libraries_url = base_url + "/osa/libraries"
-    osa_vulnerabilities_url = base_url + "/osa/vulnerabilities"
-    osa_vulnerability_comment_url = base_url + "/osa/vulnerabilities/{vulnerabilityId}/comments"
-    osa_reports_url = base_url + "/osa/reports"
 
     def __init__(self):
         self.retry = 0
 
-    def get_all_osa_scan_details_for_project(self, project_id):
+    def get_all_osa_scan_details_for_project(self, project_id=None, page=None, items_per_page=None):
         """
         Get basic scan details for all CxOSA scans associated with a specified project Id.
         v8.4.2 and up
 
         Args:
             project_id (int): Unique Id of the project
+            page (int, optional): (Number of pages (default 1)
+            items_per_page (int, optional): Number of items per page (default 100)
 
         Returns:
             list: :obj:`list` of :obj:`CxOsaScanDetail`
@@ -53,12 +47,19 @@ class OsaAPI(object):
         """
         osa_scan_details = None
 
-        if project_id:
-            self.osa_scans_url = self.osa_scans_url + "?projectId=" + str(project_id)
+        osa_scans_url = self.base_url + "/osa/scans?projectId=" + str(project_id)
+
+        optionals = []
+        if page:
+            optionals.append("page=" + str(page))
+        if items_per_page:
+            optionals.append("items_per_page=" + str(items_per_page))
+        if optionals:
+            osa_scans_url += "&".join(optionals)
 
         headers = copy.deepcopy(AuthenticationAPI.AuthenticationAPI.auth_headers)
 
-        r = requests.get(url=self.osa_scans_url, headers=headers)
+        r = requests.get(url=osa_scans_url, headers=headers)
         if r.status_code == 200:
             a_list = r.json()
             osa_scan_details = [
@@ -117,7 +118,7 @@ class OsaAPI(object):
             scan_id (str): Unique Id of the OSA scan
 
         Returns:
-           CxOsaScanDetail:
+           :obj:`CxOsaScanDetail`
 
         Raises:
             BadRequestError:
@@ -126,8 +127,9 @@ class OsaAPI(object):
         """
         osa_scan_detail = None
 
-        self.osa_scan_by_scan_id_url = self.osa_scan_by_scan_id_url.format(scanId=scan_id)
-        r = requests.get(url=self.osa_scan_by_scan_id_url, headers=AuthenticationAPI.AuthenticationAPI.auth_headers)
+        osa_scan_by_scan_id_url = self.base_url + "/osa/scans/{scanId}".format(scanId=scan_id)
+
+        r = requests.get(url=osa_scan_by_scan_id_url, headers=AuthenticationAPI.AuthenticationAPI.auth_headers)
 
         if r.status_code == 200:
             a_dict = r.json()
@@ -190,7 +192,9 @@ class OsaAPI(object):
         )
         headers.update({"Content-Type": m.content_type})
 
-        r = requests.post(url=self.osa_scans_url, headers=headers, data=m)
+        osa_scans_url = self.base_url + "/osa/scans" + "?projectId=" + str(project_id)
+
+        r = requests.post(url=osa_scans_url, headers=headers, data=m)
         if r.status_code == 202:
             scan_id = r.json().get("scanId")
         elif r.status_code == http.HTTPStatus.BAD_REQUEST:
@@ -222,7 +226,9 @@ class OsaAPI(object):
         """
         file_extensions = None
 
-        r = requests.get(url=self.osa_file_extensions_url, headers=AuthenticationAPI.AuthenticationAPI.auth_headers)
+        osa_file_extensions_url = self.base_url + "/osa/fileextensions"
+
+        r = requests.get(url=osa_file_extensions_url, headers=AuthenticationAPI.AuthenticationAPI.auth_headers)
         if r.status_code == 200:
             file_extensions = r.text
         elif r.status_code == http.HTTPStatus.BAD_REQUEST:
@@ -256,10 +262,9 @@ class OsaAPI(object):
         """
         licenses = None
 
-        if scan_id:
-            self.osa_licenses_url += "?scanId=" + str(scan_id)
+        osa_licenses_url = self.base_url + "/osa/licenses" + "?scanId=" + str(scan_id)
 
-        r = requests.get(url=self.osa_licenses_url, headers=AuthenticationAPI.AuthenticationAPI.auth_headers)
+        r = requests.get(url=osa_licenses_url, headers=AuthenticationAPI.AuthenticationAPI.auth_headers)
         if r.status_code == 200:
             a_list = r.json()
             licenses = [
@@ -290,13 +295,15 @@ class OsaAPI(object):
 
         return licenses
 
-    def get_osa_scan_libraries(self, scan_id):
+    def get_osa_scan_libraries(self, scan_id, page=None, items_per_page=None):
         """
         Get all the used libraries details for the specified CxOSA scan Id.
         Supported v8.6.0 and up
 
         Args:
-            scan_id （str): Unique Id of the OSA scan
+            scan_id (str): Unique Id of the OSA scan
+            page (int, optional): (Number of pages (default 1)
+            items_per_page (int, optional): Number of items per page (default 100)
 
         Returns:
             :obj:`list` of :obj:`CxOsaLibrary`
@@ -308,10 +315,17 @@ class OsaAPI(object):
         """
         libraries = None
 
-        if scan_id:
-            self.osa_libraries_url = self.osa_libraries_url + "?scanId=" + str(scan_id)
+        osa_libraries_url = self.base_url + "/osa/libraries" + "?scanId=" + str(scan_id)
 
-        r = requests.get(url=self.osa_libraries_url, headers=AuthenticationAPI.AuthenticationAPI.auth_headers)
+        optionals = []
+        if page:
+            optionals.append("page=" + str(page))
+        if items_per_page:
+            optionals.append("items_per_page=" + str(items_per_page))
+        if optionals:
+            osa_libraries_url += "&".join(optionals)
+
+        r = requests.get(url=osa_libraries_url, headers=AuthenticationAPI.AuthenticationAPI.auth_headers)
 
         if r.status_code == 200:
             a_list = r.json()
@@ -369,14 +383,16 @@ class OsaAPI(object):
 
         return libraries
 
-    def get_osa_scan_vulnerabilities_by_id(self, scan_id, library_id=None, state_id=None, comment=None, since=None,
-                                           until=None):
+    def get_osa_scan_vulnerabilities_by_id(self, scan_id, page=None, items_per_page=None, library_id=None,
+                                           state_id=None, comment=None, since=None, until=None):
         """
         Get all the vulnerabilities for the specified CxOSA scan Id.
         v8.4.2 and up
 
         Args:
             scan_id (str): Unique Id of the OSA scan
+            page (int, optional): (Number of pages (default 1)
+            items_per_page (int, optional): Number of items per page (default 100)
             library_id (str, optional): Filter by Library Id(s)
             state_id (int, optional):  Filter by State Id(s)
             comment (str, optional): Filter by Comment text
@@ -393,20 +409,31 @@ class OsaAPI(object):
 
         """
         vulnerabilities = None
-        if scan_id:
-            self.osa_vulnerabilities_url += "?scanId=" + str(scan_id)
-            if library_id:
-                self.osa_vulnerabilities_url += "&libraryId=" + library_id
-            if state_id:
-                self.osa_vulnerabilities_url += "&stateId=" + str(state_id)
-            if comment:
-                self.osa_vulnerabilities_url += "&comment=" + comment
-            if since:
-                self.osa_vulnerabilities_url += "&since=" + str(since)
-            if until:
-                self.osa_vulnerabilities_url += "&until=" + str(until)
 
-        r = requests.get(url=self.osa_vulnerabilities_url, headers=AuthenticationAPI.AuthenticationAPI.auth_headers)
+        osa_vulnerabilities_url = self.base_url + "/osa/vulnerabilities"
+
+        if scan_id:
+            osa_vulnerabilities_url += "?scanId=" + str(scan_id)
+
+            optionals = []
+            if page:
+                optionals.append("page=" + str(page))
+            if items_per_page:
+                optionals.append("items_per_page=" + str(items_per_page))
+            if library_id:
+                optionals.append("libraryId=" + library_id)
+            if state_id:
+                optionals.append("stateId=" + str(state_id))
+            if comment:
+                optionals.append("comment=" + comment)
+            if since:
+                optionals.append("since=" + str(since))
+            if until:
+                optionals.append("until=" + str(until))
+            if optionals:
+                osa_vulnerabilities_url += "&".join(optionals)
+
+        r = requests.get(url=osa_vulnerabilities_url, headers=AuthenticationAPI.AuthenticationAPI.auth_headers)
         if r.status_code == 200:
             a_list = r.json()
             vulnerabilities = [
@@ -482,13 +509,13 @@ class OsaAPI(object):
         """
         comment = None
 
-        if vulnerability_id and project_id:
-            self.osa_vulnerability_comment_url = self.osa_vulnerability_comment_url.format(
+        osa_vulnerability_comment_url = self.base_url + "/osa/vulnerabilities/{vulnerabilityId}/comments".format(
                 vulnerabilityId=vulnerability_id
             )
-            self.osa_vulnerability_comment_url += "?projectId=" + str(project_id)
 
-        r = requests.get(url=self.osa_vulnerability_comment_url,
+        osa_vulnerability_comment_url += "?projectId=" + str(project_id)
+
+        r = requests.get(url=osa_vulnerability_comment_url,
                          headers=AuthenticationAPI.AuthenticationAPI.auth_headers)
 
         if r.status_code == 200:
@@ -522,7 +549,7 @@ class OsaAPI(object):
             scan_id (str): Unique Id of the OSA scan
 
         Returns:
-             CxOsaSummaryReport
+             :obj:`CxOsaSummaryReport`
 
         Raises:
             BadRequestError:
@@ -531,10 +558,9 @@ class OsaAPI(object):
         """
         report = None
 
-        if scan_id:
-            self.osa_reports_url += "?scanId=" + str(scan_id)
+        osa_reports_url = self.base_url + "/osa/reports" + "?scanId=" + str(scan_id)
 
-        r = requests.get(url=self.osa_reports_url, headers=AuthenticationAPI.AuthenticationAPI.auth_headers)
+        r = requests.get(url=osa_reports_url, headers=AuthenticationAPI.AuthenticationAPI.auth_headers)
 
         if r.status_code == 200:
             a_dict = r.json()
